'''
Contains all the commonly used functions and data
useful for multiple tmd modules.
'''

term_dict = {'x': 0,
             'y': 1,
             'z': 2}

tree_type = {1: 'soma',
             2: 'axon',
             3: 'basal',
             4: 'apical'}
