"""
    tmd : Neuron class

    Author: Lida Kanari

    Ecole Polytechnique Federale de Lausanne
    Blue Brain Project

    (c) 2014-2015. All rights reserved.
"""

from tmd.Population import Population
